#include <pwd.h>
#include <grp.h>
#include <stdio.h>
#include <errno.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "make_path.h"
#include "get_stat.h"
#include "recurse.h"
#include "unused.h"

char r_flag;
char *f_flag = "chown";
char H_flag;
char v_flag;
int (*chown_func)(const char *pathname, uid_t owner, gid_t group);
long gid;
long uid;

int change(const char *path, void *p) {
	char *name = (char *)p;

	if (chown_func(path, uid, gid)) {
		if (f_flag)
			fprintf(stderr, "chown: unable to chown %s: %s\n", path, strerror(errno));

		return 1;
	}

	if (v_flag)
		printf("chown: %s: changed to %s\n", path, name);

	return 0;
}

void get_owner(const char *arg) {
	char *group = strchr(arg, ':');

	char g_flag = 1;
	char u_flag = 1;

	if (group == arg)
		u_flag = 0;

	else if (!group)
		g_flag = 0;

	if (g_flag) {
		group[0] = '\0';
		group++;

		struct group *grp = getgrnam(group);
		if (!grp) {
			if (f_flag)
				fprintf(stderr, "chown: invalid group: %s\n", group);

			exit(1);
		}

		gid = grp->gr_gid;
	}

	if (u_flag) {
		struct passwd *pwd = getpwnam(arg);
		if (!pwd) {
			if (f_flag)
				fprintf(stderr, "chown: invalid user: %s\n", arg);

			exit(1);
		}

		uid = pwd->pw_gid;
	}
}

int main(int argc, char **argv) {
	chown_func = lchown;

	int opt;
	while ((opt = getopt(argc, argv, "RfHv")) != -1) {
		switch (opt) {
			case 'R':
				r_flag = 1;
				break;

			case 'f':
				f_flag = NULL;
				break;

			case 'H':
				H_flag = 1;
				chown_func = chown;
				break;

			case 'v':
				v_flag = 1;
				break;

			default:
				printf("chown [RfHf] USER[:[GRP]] [file1 file2...]\n\t-H Symbolic link\n\t-R Recursive\n\t-f Silent\n\t-v Verbose\n");
				return 0;
		}
	}

	argv += optind;
	argc -= optind;

	if (argc == 0) {
		fprintf(stderr, "chown: missing operand\n");
		return 1;
	}

	gid = -1;
	uid = -1;
	get_owner(argv[0]);

	int ret = 0;
	for (int i = 1; i < argc; i++) {
		if (r_flag) {
			if (mu_recurse(f_flag, H_flag, argv[i], argv[0], change, change))
				ret = 1;
		}

		else
			ret = change(argv[i], argv[0]);
	}

	return ret;
}
